# vim: set noexpandtab tabstop=2 shiftwidth=2 softtabstop=-1 fileencoding=utf-8:

import sys
f, numthreads, thresh_methods=snakemake.input[0], snakemake.params.numthreads, snakemake.params.method

import scanpy as sc
sc.set_figure_params(color_map='viridis', frameon=False)

import os
if os.path.isdir(f):
	x=sc.read_10x_mtx(f)
elif f.endswith('.h5'):
	x=sc.read_10x_h5(f)
elif f.endswith('.h5ad'):
	x=sc.read(f)
x.var_names_make_unique()
print(vars(x))

import dropkick as dk
x=dk.recipe_dropkick(x, n_hvgs=None, X_final='raw_counts')
dk.qc_summary(x, save_to=snakemake.output[0])
print(vars(x))

x_classifier=dk.dropkick(x, n_jobs=numthreads, thresh_methods=thresh_methods)

dk.score_plot(x, save_to=snakemake.output[1])
dk.coef_inventory(x)
dk.coef_plot(x, save_to=snakemake.output[2])

x.obs.dropkick_label=x.obs.dropkick_label.astype('string') # A potential bug without this conversion
sc.write(filename=snakemake.output[3], adata=x)
x.obs['barcode']=x.obs.index
x.obs.to_csv(snakemake.output[4], sep='\t', index=False)
x.var['symbol']=x.var.index
x.var.to_csv(snakemake.output[5], sep='\t', index=False)



#!/usr/bin/env python3
# vim: set noexpandtab tabstop=2 shiftwidth=2 softtabstop=-1 fileencoding=utf-8:

import sys
import click
import datetime
import shutil
import subprocess
import os
from pathlib import Path

def runcmd(cmdstr):
	try:
		cmdstr=' '.join(cmdstr)

		click.echo(f'Start running: {cmdstr}', file=sys.stderr)
		exitcode=subprocess.call(cmdstr, shell=True, executable=shutil.which('bash'))
		click.echo(f'Finish running: {cmdstr}', file=sys.stderr)

		if exitcode!=0:
			click.echo(f'Error: {cmdstr} failed. Exit code: {exitcode}', file=sys.stderr)
			sys.exit(-1)

	except OSError as e:
		click.echo("Execution failed: ", e, file=sys.stderr)
		sys.exit(-1)

	return exitcode

CONTEXT_SETTINGS=dict(help_option_names=['-h', '--help'])
@click.command(context_settings=CONTEXT_SETTINGS)
@click.option('-d', '--outdir', type=click.Path(), default='.', help='Outdir. (default: ".")')
@click.option('-c', '--configfile', type=click.Path(exists=False, resolve_path=True), help='Configuration file in the YAML format.')
@click.option('-r', '--rule', type=click.STRING, help='Force to re-run a rule and its downstream. Available: soupx, dropkick, h5subset, doubletfinder.')
@click.option('-t', '--numthreads', type=click.INT, default=4, help='Number of threads. (default: 4)')
@click.option('-D', '--dagonly', is_flag=True, help='Generate the DAG of jobs only. Useful to update dag.pdf.')
@click.option('-R', '--reportonly', is_flag=True, help='Generate the report only. Useful to update report.html.')
@click.option('-S', '--summaryonly', is_flag=True, help='Generate the detailed summary only. Useful to update summary.txt.')
@click.option('-n', '--dryrun', is_flag=True, help='Dry-run.')
@click.argument('samplefile', type=click.Path(exists=False, resolve_path=True))
@click.version_option()
def main(configfile, rule, outdir, numthreads, dagonly, reportonly, summaryonly, dryrun, samplefile):
	"""
cellqc: standardized quality control pipeline of single-cell RNA-Seq data.

SAMPLEFILE is a headed tab-delimited sample file for samples in the below format.

\b
```
sample<TAB>cellranger[<TAB>nreaction]
```

\b
Example:
  outdir=$(mktemp -d -u)
  cellqc -d "$outdir" -t 8 -n -- samples.txt # save default parameters to outdir/config.yaml
  ## copy from outdir/config.yaml and edit config.yaml
  cellqc -d "$outdir" -t 8 -c config.yaml -- samples.txt

\b
Note:
  1. The directory of the `cellranger` in SAMPLEFILE should exist.

\b
Date: 2023/02/15
Authors: Jin Li <lijin.abc@gmail.com>
	"""
	nowtimestr=datetime.datetime.now().strftime('%y%m%d_%H%M%S')
	Path(outdir).mkdir(parents=True, exist_ok=True)
	absdir=Path(__file__).parent
	os.chdir(outdir)

	cmdstr=[
		f"snakemake",
		f"-s {absdir}/Snakefile",
		f"-d {outdir}",
		f"-j {numthreads}",
		f"-C samplefile='{samplefile}' outdir='{outdir}' configfile='{configfile}' nowtimestr='{nowtimestr}'",
		]
	if rule:
		cmdstr+=[f"-R {rule}"]
	if configfile:
		cmdstr+=[f"--configfile {configfile}"]

	if dryrun:
		cmdstr+=[f"-n -p"]
		runcmd(cmdstr)

	else:
		cmdrun=cmdstr+[
			f"-r -p --debug-dag",
			f"--stats Snakefile_{nowtimestr}.stats",
			]
		cmdreport=cmdstr+[
			f"--report report_{nowtimestr}.html",
			]
		cmddag=cmdstr+[
			f"--dag | tee dag_{nowtimestr}.dot | dot -Tpdf > dag_{nowtimestr}.pdf",
			]
		cmdsummary=cmdstr+[
			f"-D -c1 | tee summary_{nowtimestr}.txt",
			]

		if dagonly:
			runcmd(cmddag)
		elif reportonly:
			runcmd(cmdreport)
		elif summaryonly:
			runcmd(cmdsummary)
		else:
			runcmd(cmdrun)
			runcmd(cmddag)
			runcmd(cmdreport)
			runcmd(cmdsummary)

	return 0

if __name__ == "__main__":
	sys.exit(main())



# vim: set noexpandtab tabstop=2 shiftwidth=2 softtabstop=-1 fileencoding=utf-8:

import os
import sys
import pandas as pd
from pathlib import Path
import mimetypes
import base64
from jinja2 import Environment, FileSystemLoader

# From snakemake/report/__init__.py
def data_uri(data, filename, encoding="utf8", mime="text/plain"):
	"""Craft a base64 data URI from file with proper encoding and mimetype."""
	data=base64.b64encode(data)
	uri="data:{mime};charset={charset};filename={filename};base64,{data}" "".format(
		filename=filename, mime=mime, charset=encoding, data=data.decode("utf-8")
		)
	return uri

def mime_from_file(file):
	mime, encoding=mimetypes.guess_type(file)
	if mime is None:
		mime="text/plain"
		print(f"Could not detect mimetype for {file}, assuming text/plain.", file=sys.stderr)
	return mime, encoding

def data_uri_from_file(file, defaultenc="utf8"):
	"""Craft a base64 data URI from file with proper encoding and mimetype."""
	if isinstance(file, Path):
		file=str(file)
	mime, encoding=mime_from_file(file)
	if encoding is None:
		encoding=defaultenc
	with open(file, "rb") as f:
		return data_uri(f.read(), os.path.basename(file), encoding, mime)

# From snakemake/report/data/common.py
def get_resource_as_string(path):
	return open(Path(__file__).parent / "template" / path).read()

env=Environment(loader=FileSystemLoader(Path(__file__).parent / "template"))
env.filters["get_resource_as_string"]=get_resource_as_string
template=env.get_template("index.html.jinja2")

## Cellranger metrics summary
tmp=[]
for k, v in snakemake.params.samples['cellranger'].to_dict().items():
	vf=os.path.join(snakemake.params.sampledir, f"{v}/metrics_summary.csv")
	if os.path.exists(vf):
		x=pd.read_csv(vf)
		x.insert(0, 'sampleid', k)
		tmp+=[x]
_cellrangersummary=pd.concat(tmp, ignore_index=True).to_html() if len(tmp)>0 else None

## SoupX
tmp=[pd.read_csv(f, sep='\t', header=0) for f in snakemake.input.soupxrhoEst]
_soupxrhoEst=pd.concat(tmp, ignore_index=True).to_html() if len(tmp)>0 else None
_soupxrho={f : data_uri_from_file(f) for f in snakemake.input.soupxrho}

## Dropkick
tmp=[pd.read_csv(f, sep='\t', header=0) for f in snakemake.input.dropkickstat]
_dropkickstat=pd.concat(tmp, ignore_index=True).to_html() if len(tmp)>0 else None
_dropkicksummary={f : data_uri_from_file(f) for f in snakemake.input.dropkicksummary} if len(snakemake.input.dropkicksummary)>0 else None
_dropkickscore={f : data_uri_from_file(f) for f in snakemake.input.dropkickscore} if len(snakemake.input.dropkickscore)>0 else None

## Filterbycounts
tmp=[pd.read_csv(f'{dir}/filter_ncell.txt', sep='\t', header=0) for dir in snakemake.input.filterbycountdir]
_filterbycountncell=pd.concat(tmp, ignore_index=True).to_html() if len(tmp)>0 else None
_filterbycountpltbf={dir : data_uri_from_file(f'{dir}/feature_bf.pdf') for dir in snakemake.input.filterbycountdir}
_filterbycountpltaf={dir : data_uri_from_file(f'{dir}/feature_af.pdf') for dir in snakemake.input.filterbycountdir}

## DoubletFinder
pK=snakemake.params.pK
tmp=[pd.read_csv(f'{dir}/doublet_ratio.txt', sep='\t', header=0) for dir in snakemake.input.doubletfinderdir]
_doubletratio=pd.concat(tmp, ignore_index=True).to_html() if len(tmp)>0 else None
_doubletpannviolin={dir : data_uri_from_file(f'{dir}/pANN_violin_pK{pK}.pdf') for dir in snakemake.input.doubletfinderdir}
_doublettsne={dir : data_uri_from_file(f'{dir}/tsne_doublet_pK{pK}.pdf') for dir in snakemake.input.doubletfinderdir}
_doubletumap={dir : data_uri_from_file(f'{dir}/umap_doublet_pK{pK}.pdf') for dir in snakemake.input.doubletfinderdir}

## scPred
_scpredcontingency={dir : pd.read_csv(f'{dir}/contingency.txt', sep='\t', header=0).to_html() for dir in snakemake.input.scpreddir} if len(snakemake.input.scpreddir)>0 else None
_scpredfeaturemax={dir : data_uri_from_file(f'{dir}/feature_scpred_max.png') for dir in snakemake.input.scpreddir} if len(snakemake.input.scpreddir)>0 else None
_scpredpredictumap={dir : data_uri_from_file(f'{dir}/predict_umap.png') for dir in snakemake.input.scpreddir} if len(snakemake.input.scpreddir)>0 else None

## Render report file from a Jinja template
outfile=snakemake.output[0]
with open(outfile, mode="w", encoding="utf-8") as f:
	f.write(
		template.render(
			cellrangersummary=_cellrangersummary,
			soupxrhoEst=_soupxrhoEst,
			soupxrho=_soupxrho,
			dropkickstat=_dropkickstat,
			dropkicksummary=_dropkicksummary,
			dropkickscore=_dropkickscore,
			filterbycountncell=_filterbycountncell,
			filterbycountpltbf=_filterbycountpltbf,
			filterbycountpltaf=_filterbycountpltaf,
			doubletratio=_doubletratio,
			doubletpannviolin=_doubletpannviolin,
			doublettsne=_doublettsne,
			doubletumap=_doubletumap,
			scpredcontingency=_scpredcontingency,
			scpredfeaturemax=_scpredfeaturemax,
			scpredpredictumap=_scpredpredictumap,
			)
		)



#!/usr/bin/env python3

import os
import io
import re
import ast

from setuptools import setup, find_packages

DEPENDENCIES=['click', 'snakemake', 'pygraphviz', 'Jinja2']
EXCLUDE_FROM_PACKAGES=["contrib", "docs", "tests*"]
CURDIR=os.path.abspath(os.path.dirname(__file__))

with io.open(os.path.join(CURDIR, "README.md"), "r", encoding="utf-8") as readme_file:
    readme=readme_file.read()

def get_version():
    main_file=os.path.join(CURDIR, "cellqc", "__init__.py")
    _version_re=re.compile(r"__version__\s+=\s+(?P<version>.*)")
    with open(main_file, "r", encoding="utf8") as f:
        match=_version_re.search(f.read())
        version=match.group("version") if match is not None else '"unknown"'
    return str(ast.literal_eval(version))

setup(
    name="cellqc",
    author="Jin Li",
    author_email="lijin.abc@gmail.com",
    python_requires=">=3.6",
    description="Cellqc standardizes the qualiy control of single-cell RNA-Seq (scRNA) data to render clean feature count matrices.",
    install_requires=DEPENDENCIES,
    packages=find_packages(exclude=EXCLUDE_FROM_PACKAGES),
    long_description=readme,
    long_description_content_type="text/markdown",
    include_package_data=True,
    keywords=[],
    scripts=[],
    setup_requires=[],
    entry_points={
        "console_scripts": [
            "cellqc=cellqc.cellqc:main",
            ]
        },
    url="https://github.com/lijinbio/cellqc",
    version=get_version(),
    zip_safe=False,
    license="MIT license",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
    ],
)
